package cn.ninegame.genericframework.basic;

import android.util.Log;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.osgi.framework.FrameworkEvent;
import org.osgi.framework.FrameworkListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 框架服务层， 可以通过该类获取框架通用的管理类
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/2/29
 */
public class FrameworkService {

    static final String FRAMEWORK_VERSION = "0.0.1";

    private static ClassNotFoundInterceptorCallback classNotFoundCallback;

    public static final String STAT_SERVICE = "stat";
    public static final String EXCEPTION_SERVICE = "exception";
    public static final String SECURITY_SERVICE = "security";
    public static final String CACHE_SERVICE = "cache";
    public static final String LOG_SERVICE = "log";

    private static FrameworkService mInstance;
    private HashMap<String, Object> mServices;

    //  是否debug模式
    boolean mIsDebug;

    /**
     * debug outputs from class loading ?
     */
    static boolean DEBUG_CLASSLOADING;

    /**
     * location -> bundle.
     */
    static Map<String, Bundle> bundles = new ConcurrentHashMap<String, Bundle>();

    /**
     * bundle listeners.
     */
    static List<BundleListener> bundleListeners = new ArrayList<BundleListener>();

    /**
     * synchronous bundle listeners.
     */
    static List<BundleListener> syncBundleListeners = new ArrayList<BundleListener>();

    /**
     * framework listeners.
     */
    static List<FrameworkListener> frameworkListeners = new ArrayList<FrameworkListener>();

    /**
     * system ClassLoader
     */
    static ClassLoader systemClassLoader;


    public static FrameworkService getInstance(){
        if (mInstance == null){
            synchronized (FrameworkService.class){
                if (mInstance == null){
                    mInstance = new FrameworkService();
                }
            }
        }
        return mInstance;
    }

    private FrameworkService(){
        mServices = new HashMap<String, Object>();
    }

    @SuppressWarnings("unchecked")
    public <T> T getService(Class<T> serviceClass){
        String serviceName = getServiceName(serviceClass);
        return serviceName != null ? (T)getService(serviceName) : null;
    }

    public Object getService(String name){
        Object objService = mServices.get(name);
        if (objService == null){
            objService = createServiceInstance(name);
            mServices.put(name, objService);
        }
        return objService;
    }

    /**
     * 创建Service实例
     * @param serviceName
     * @return
     */
    private Object createServiceInstance(String serviceName){
        if (STAT_SERVICE.equals(serviceName)){
            return new StatManager();
        } else if (EXCEPTION_SERVICE.equals(serviceName)){
            return new ExceptionManager();
        } else if (SECURITY_SERVICE.equals(serviceName)){
            return new SecurityManager();
        } else if(CACHE_SERVICE.equals(serviceName)){
            return new CacheManager();
        }else if(CACHE_SERVICE.equals(serviceName)){
            return new CacheManager();
        } else if(LOG_SERVICE.equals(serviceName)){
            return new LogManager();
        } else {
            throw new RuntimeException("services name not available");
        }
    }

    /**
     * 获取框架服务名
     * @param serviceClass
     * @return 返回服务名或者null
     */
    public String getServiceName(Class<?> serviceClass){
        String serviceName;
        if (serviceClass == StatManager.class){
            serviceName = STAT_SERVICE;
        } else if (serviceClass == ExceptionManager.class){
            serviceName = EXCEPTION_SERVICE;
        } else if (serviceClass == SecurityManager.class){
            serviceName = SECURITY_SERVICE;
        } else if(serviceClass == CacheManager.class){
            serviceName = CACHE_SERVICE;
        } else if(serviceClass == LogManager.class){
            serviceName = LOG_SERVICE;
        } else {
            serviceName = null;
        }

        return serviceName;
    }

    public boolean IsDebug() {
        return mIsDebug;
    }

    public void setDebug(boolean mIsDebug) {
        this.mIsDebug = mIsDebug;
    }


    public static ClassNotFoundInterceptorCallback getClassNotFoundCallback() {
        return classNotFoundCallback;
    }

    public static void setClassNotFoundCallback(ClassNotFoundInterceptorCallback classNotFoundCallback) {
        FrameworkService.classNotFoundCallback = classNotFoundCallback;
    }

    /**
     * notify all bundle listeners.
     *
     * @param state the new state.
     * @param bundle the bundle.
     */
    static void notifyBundleListeners(final int state, final Bundle bundle) {
        if (syncBundleListeners.isEmpty() && bundleListeners.isEmpty()) {
            return;
        }

        final BundleEvent event = new BundleEvent(state, bundle);

        // inform the synchrounous bundle listeners first ...
        final BundleListener[] syncs = (BundleListener[]) syncBundleListeners.toArray(new BundleListener[syncBundleListeners.size()]);

        for (int i = 0; i < syncs.length; i++) {
            syncs[i].bundleChanged(event);
        }

        if (bundleListeners.isEmpty()) {
            return;
        }

        final BundleListener[] asyncs = (BundleListener[]) bundleListeners.toArray(new BundleListener[bundleListeners.size()]);
        for (int i = 0; i < asyncs.length; i++) {
            asyncs[i].bundleChanged(event);
        }
    }

    public static void addFrameworkListener(FrameworkListener listener) {
        frameworkListeners.add(listener);
    }

    public static void removeFrameworkListener(FrameworkListener listener) {
        frameworkListeners.remove(listener);
    }

    static void addBundleListener(BundleListener listener) {
        bundleListeners.add(listener);
    }

    static void removeBundleListener(BundleListener listener) {
        bundleListeners.remove(listener);
    }

    /**
     * notify all framework listeners.
     *
     * @param state the new state.
     * @param bundle the bundle.
     * @param throwable a throwable.
     */
    static void notifyFrameworkListeners(final int state, final Bundle bundle, final Throwable throwable) {

        if (frameworkListeners.isEmpty()) {
            return;
        }

        final FrameworkEvent event = new FrameworkEvent(state, bundle, throwable);

        final FrameworkListener[] listeners = (FrameworkListener[]) frameworkListeners.toArray(new FrameworkListener[frameworkListeners.size()]);

        for (int i = 0; i < listeners.length; i++) {
            final FrameworkListener listener = listeners[i];

            listener.frameworkEvent(event);
        }
    }

    public static void handleException(Exception ex){
        ExceptionManager exceptionManager = FrameworkService.getInstance().getService(ExceptionManager.class);
        if(exceptionManager!=null){
            exceptionManager.handleException(ex);
        }
    }

    public static void addStat(String category, HashMap<String, String> bodies){
        StatManager statManager = FrameworkService.getInstance().getService(StatManager.class);
        if(statManager!=null){
            statManager.addStat(category,bodies);
        }
    }

    public static void addDebugLog(String tag, ILogHandler.LogLevel logLevel,String msg){
        if(FrameworkService.getInstance().mIsDebug && !(logLevel== ILogHandler.LogLevel.error)){
            LogManager logManager = FrameworkService.getInstance().getService(LogManager.class);
            if(logManager!=null){
                logManager.log(tag,logLevel,msg);
            }
        }else if(logLevel== ILogHandler.LogLevel.error){
            LogManager logManager = FrameworkService.getInstance().getService(LogManager.class);
            if(logManager!=null){
                logManager.log(tag,logLevel,msg);
            }
        }
    }

    public static void addDebugLog(String tag, String message,Throwable throwable){
        LogManager logManager = FrameworkService.getInstance().getService(LogManager.class);
        if(logManager!=null){
            logManager.log(tag,message,throwable);
        }
    }
}
