package cn.ninegame.genericframework.basic;

import android.content.Context;
import android.os.Bundle;

import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.ui.BaseDialogFragment;
import cn.ninegame.genericframework.ui.BaseFragment;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-03-18
 * Author      : lihq@ucweb.com
 * Changed     : zhonghui.jzh
 * Description : BaseController承担了组件间消息通信的职责，是PAC模式中的Control。同时也是模块对外的接口
 * ****************************************************************************
 */
public abstract class GundamCommander implements IController,INotify {
    Environment mEnv;
    private IModuleEntry mModuleEntry;

    public GundamCommander() {
    }

    @Override
    public void onInit() {
        registerNotification();
    }

    @Override
    public void setEnvironment(Environment environment) {
        mEnv = environment;
    }

    @Override
    public Environment getEnvironment() {
        return mEnv;
    }

    @Override
    public boolean isPermanent() {
        return true;
    }

    @Override
    public void onNotify(Notification notification) {

    }

    public void sendMessage(int messageId){
        if(mEnv != null){
            mEnv.sendMessage(messageId);
        }
    }

    public void sendMessage(int messageId, Bundle messageData){
        if(mEnv != null){
            mEnv.sendMessage(messageId, messageData);
        }
    }

    public void sendMessageForResult(int messageId, Bundle messageData,IResultListener listener){
        if(mEnv != null){
            mEnv.sendMessageForResult(messageId, messageData, listener);
        }
    }

    public Bundle sendMessageSync(int messageId, Bundle messageData) {
        if(mEnv != null){
            return mEnv.sendMessageSync(messageId,messageData);
        }
        return null;
    }

    @Override
    public void handleMessage(GMessage message) {

    }

    @Override
    public Bundle handleMessageSync(GMessage message) {
        return null;
    }

    public void sendNotification(int notificationID, Bundle extraData){
        if(mEnv != null){
            mEnv.sendNotification(Notification.obtain(notificationID, extraData));
        }
    }

    public void sendNotification(String notificationName, Bundle extraData){
        if(mEnv != null){
            mEnv.sendNotification(notificationName, extraData);
        }
    }

    public void registerNotification(int notificationId, INotify notify){
        if(mEnv != null){
            mEnv.registerNotificationWithId(notificationId, notify);
        }
    }

    public void unregisterNotification(int notificationId, INotify notify){
        if(mEnv != null){
            mEnv.registerNotificationWithId(notificationId, notify);
        }
    }

    public void registerNotification(String notificationName, INotify notify){
        if(mEnv != null){
            mEnv.registerNotification(notificationName, notify);
        }
    }

    public void unregisterNotification(String notificationName, INotify notify){
        if(mEnv != null){
            mEnv.registerNotification(notificationName, notify);
        }
    }

    public void registerNotification() {
        RegisterNotification[] notifications = getNotification();

        for (int i = 0; i < notifications.length; i++) {
            getEnvironment().registerNotification(notifications[i].name(), this);
        }
    }

    public void unregisterNotification(){
        RegisterNotification[] notifications = getNotification();

        for (int i = 0; i < notifications.length; i++) {
            getEnvironment().unregisterNotification( notifications[i].name(), this);
        }
    }

    private RegisterNotification[] getNotification(){
        RegisterNotification[] notifications = {};
        try {
            RegisterNotifications annotation = this.getClass().getAnnotation(RegisterNotifications.class);

            if (annotation != null) {
                notifications = annotation.value();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return notifications;
    }

    @Override
    public void setModuleEntry(IModuleEntry entry) {
        if (mModuleEntry != null) {
            throw new IllegalArgumentException("Module entry has already set!");
        }
        mModuleEntry = entry;
    }

    @Override
    public Context getContext() {
        return mModuleEntry.getContext();
    }

    public BaseFragment loadFragment(String clzName){
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("loadFragment,  mModuleEntry is null, className = %s", getClass().getName()));
        }
        BaseFragment fragment = mModuleEntry.loadFragment(clzName);
        fragment.setEnvironment(mEnv);
        return fragment;
    }

    public BaseDialogFragment loadDialogFragment(String clzName){
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("loadDialogFragment,  mModuleEntry is null, className = %s", getClass().getName()));
        }
        BaseDialogFragment fragment = mModuleEntry.loadDialogFragment(clzName);
        fragment.setEnvironment(mEnv);
        return fragment;
    }

    @Override
    public String getModuleRootPath(){
        return mModuleEntry.getModuleRootPath();
    }

    @Override
    public String getModuleDataPath(){
        return mModuleEntry.getModuleDataPath();
    }

    @Override
    public String getModuleSoPath(){
        return mModuleEntry.getModuleSoPath();
    }
}
