package cn.ninegame.genericframework.module;

import android.util.Log;

import java.io.File;
import java.io.IOException;

import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.tools.FileUtil;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 *  Module 签名校验保存
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/3/31
 */
public class ModuleSignature {

    private static final String TAG = "ModuleSignature";

    private String mModuleId;

    public ModuleSignature(String moduleId){
        this.mModuleId = moduleId;
    }

    /**
     * 获取 module 签名
     * @return
     */
    public String getSignature(){
        File file = ModulePathManager.getInstance().getModuleSignatureFile(mModuleId);
        if (file.exists()){
            return  FileUtil.readProperty(file, mModuleId);
        } else {
            return null;
        }
    }

    /**
     * 更新保存Module的签名信息
     * @param signature
     */
    public void updateSignature(String signature){

        Log.d(TAG, String.format("updateSignature, moduleId=%s, signature=%s", mModuleId, signature));

        File file = ModulePathManager.getInstance().getModuleSignatureFile(mModuleId);
        if (!file.exists()){
            File parent = file.getParentFile();
            if (parent != null && !parent.exists()) {
                parent.mkdirs();
            }
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        FileUtil.writeProperty(file, mModuleId, signature);
    }
}
