package cn.ninegame.genericframework.tools;

import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.os.Parcelable;
import android.os.PersistableBundle;
import android.util.Size;
import android.util.SizeF;
import android.util.SparseArray;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: Bundle链式构造器，使用方法</p>
 * <p>Bundle bundle = new BundleBuilder().putString("k1","v1").putInt("k2", v2).create();</p>
 *
 * <p>Copyright: Copyright (c) 2015</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2015/12/3
 */
public class BundleBuilder{
    private Bundle bundle;

    public BundleBuilder() {
        bundle = new Bundle();
    }

    public BundleBuilder(Bundle bundle) {
        this.bundle = bundle;
    }

    /**
     * 返回Bundle
     * @return
     */
    public Bundle create() {
        return bundle;
    }

    public BundleBuilder putAll(Bundle bundle) {
        this.bundle.putAll(bundle);
        return this;
    }

    public BundleBuilder putAll(PersistableBundle bundle) {
        if (Build.VERSION.SDK_INT >= 21) {
            this.bundle.putAll(bundle);
        }
        return this;
    }


    public BundleBuilder putBinder(String key, IBinder value) {
        if (Build.VERSION.SDK_INT >= 18) {
            this.bundle.putBinder(key, value);
        }
        return this;
    }

    public BundleBuilder putBoolean(String key, boolean value) {
        this.bundle.putBoolean(key, value);
        return this;
    }

    public BundleBuilder putBooleanArray(String key, boolean[] value) {
        this.bundle.putBooleanArray(key, value);
        return this;
    }

    public BundleBuilder putBundle(String key, Bundle value) {
        this.bundle.putBundle(key, value);
        return this;
    }

    public BundleBuilder putByte(String key, byte value) {
        this.bundle.putByte(key, value);
        return this;
    }

    public BundleBuilder putByteArray(String key, byte[] value) {
        this.bundle.putByteArray(key, value);
        return this;
    }

    public BundleBuilder putChar(String key, char value) {
        this.bundle.putChar(key, value);
        return this;
    }

    public BundleBuilder putCharArray(String key, char[] value) {
        this.bundle.putCharArray(key, value);
        return this;
    }

    public BundleBuilder putCharSequence(String key, CharSequence value) {
        this.bundle.putCharSequence(key, value);
        return this;
    }

    public BundleBuilder putCharSequenceArray(String key, CharSequence[] value) {
        this.bundle.putCharSequenceArray(key, value);
        return this;
    }

    public BundleBuilder putCharSequenceArrayList(String key, ArrayList<CharSequence> value) {
        this.bundle.putCharSequenceArrayList(key, value);
        return this;
    }

    public BundleBuilder putFloat(String key, float value) {
        this.bundle.putFloat(key, value);
        return this;
    }

    public BundleBuilder putFloatArray(String key, float[] value) {
        this.bundle.putFloatArray(key, value);
        return this;
    }

    public BundleBuilder putIntegerArrayList(String key, ArrayList<Integer> value) {
        this.bundle.putIntegerArrayList(key, value);
        return this;
    }

    public BundleBuilder putInt(String key, int value) {
        this.bundle.putInt(key, value);
        return this;
    }

    public BundleBuilder putParcelable(String key, Parcelable value) {
        this.bundle.putParcelable(key, value);
        return this;
    }

    public BundleBuilder putParcelableArray(String key, Parcelable[] value) {
        this.bundle.putParcelableArray(key, value);
        return this;
    }

    public BundleBuilder putParcelableArrayList(String key,
                                                ArrayList<? extends Parcelable> value) {
        this.bundle.putParcelableArrayList(key, value);
        return this;
    }

    public BundleBuilder putSerializable(String key, Serializable value) {
        this.bundle.putSerializable(key, value);
        return this;
    }

    public BundleBuilder putShort(String key, short value) {
        this.bundle.putShort(key, value);
        return this;
    }

    public BundleBuilder putShortArray(String key, short[] value) {
        this.bundle.putShortArray(key, value);
        return this;
    }

    public BundleBuilder putSize(String key, Size value) {
        if (Build.VERSION.SDK_INT >= 21) {
            this.bundle.putSize(key, value);
        }
        return this;
    }

    public BundleBuilder putSizeF(String key, SizeF value) {
        if (Build.VERSION.SDK_INT >= 21) {
            this.bundle.putSizeF(key, value);
        }
        return this;
    }

    public BundleBuilder putSparseParcelableArray(String key,
                                                  SparseArray<? extends Parcelable> value) {
        this.bundle.putSparseParcelableArray(key, value);
        return this;
    }

    public BundleBuilder putString(String key, String value) {
        this.bundle.putString(key, value);
        return this;
    }

    public BundleBuilder putStringArrayList(String key, ArrayList<String> value) {
        this.bundle.putStringArrayList(key, value);
        return this;
    }

    public BundleBuilder putStringArray(String key, String[] value) {
        this.bundle.putStringArray(key, value);
        return this;
    }

    public BundleBuilder putDouble(String key, double value) {
        this.bundle.putDouble(key, value);
        return this;
    }

    public BundleBuilder putDoubleArray(String key, double[] value) {
        this.bundle.putDoubleArray(key, value);
        return this;
    }

    public BundleBuilder putIntArray(String key, int[] value) {
        this.bundle.putIntArray(key, value);
        return this;
    }

    public BundleBuilder putLong(String key, long value) {
        this.bundle.putLong(key, value);
        return this;
    }

    public BundleBuilder putLongArray(String key, long[] value) {
        this.bundle.putLongArray(key, value);
        return this;
    }

    /**
     * Bundle.getString(String, String)是API 12开始才提供的方法， 不建议直接使用。
     * @param key
     * @param defaultValue
     * @return
     */
    public String getString(String key, String defaultValue){
        if (bundle == null){
            return defaultValue;
        }

        String result = bundle.getString(key);
        if (result == null){
            result = defaultValue;
        }
        return result;
    }
}
