package cn.ninegame.genericframework.tools;

import android.text.TextUtils;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-04-07
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
public final class ReflectHelper {

    public static Object loadClass(String classFullName) {
        if (TextUtils.isEmpty(classFullName)) {
            return null;
        }

        Object object = null;
        try {
            Class<?> clazz = Class.forName(classFullName);
            if (clazz == null) {
                return null;
            }

            Constructor<?> constructor = clazz.getDeclaredConstructor();
            //insure the constructor is public
            if (!constructor.isAccessible()) {
                constructor.setAccessible(true);
            }
            object = constructor.newInstance();

        } catch (Exception e) {
            e.printStackTrace();
        }

        return object;
    }

    /**
     * 反射出有构造参数的类的实例
     *
     * @param classFullName
     * @param parameterTypes
     * @param args
     * @return
     */
    public static Object loadClass(String classFullName, Class<?>[] parameterTypes, Object... args) {
        Constructor<?> constructor = null;
        try {
            constructor = Class.forName(classFullName).getConstructor(parameterTypes);
            if (!constructor.isAccessible()) {
                constructor.setAccessible(true);
            }
            return constructor.newInstance(args);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }


    public static Field findField(Object instance, String name)
            throws NoSuchFieldException, IllegalAccessException {
        Object[] rs = getField(instance, name, true);
        if (rs == null) {
            throw new NoSuchFieldException("field:" + name);
        }
        Field field = (Field) rs[0];
        return field;
    }


    public static Method findMethod(Object instance, String name, Class<?>[] parameterTypes)
            throws NoSuchMethodException {
        for (Class clazz = instance.getClass(); clazz != null; clazz = clazz.getSuperclass()) {
            try {
                Method method = clazz.getDeclaredMethod(name, parameterTypes);

                if (!method.isAccessible()) {
                    method.setAccessible(true);
                }

                return method;
            } catch (NoSuchMethodException e) {
            }
        }
        throw new NoSuchMethodException(new StringBuilder().append("Method ").append(name).append(" with parameters ").append(Arrays.asList(parameterTypes)).append(" not found in ").append(instance.getClass()).toString());
    }

    public static Method findMethod(Class cls, String name, Class<?>[] parameterTypes)
            throws NoSuchMethodException {
        try {
            Method method = cls.getDeclaredMethod(name, parameterTypes);

            if (!method.isAccessible()) {
                method.setAccessible(true);
            }

            return method;
        } catch (NoSuchMethodException e) {
        }
        throw new NoSuchMethodException(new StringBuilder().append("Method ").append(name).append(" with parameters ").append(Arrays.asList(parameterTypes)).append(" not found in ").append(cls.getClass()).toString());
    }

    @SuppressWarnings("unchecked")
    public static <T> T getFieldValue(Object obj, String fieldName) throws IllegalAccessException, IllegalArgumentException, NoSuchFieldException {
        Object[] rs = getField(obj, fieldName, true);
        if (rs == null) {
            throw new NoSuchFieldException("field:" + fieldName);
        }
        Field field = (Field) rs[0];
        Object targetObj = rs[1];
        return (T) field.get(targetObj);
    }

    public static void setField(Object obj, String fieldName, Object val) throws IllegalAccessException, IllegalArgumentException, NoSuchFieldException {
        Object[] rs = getField(obj, fieldName, true);
        if (rs == null) {
            throw new NoSuchFieldException("field:" + fieldName);
        }
        Field field = (Field) rs[0];
        Object targetObj = rs[1];
        field.set(targetObj, val);
    }

    private static Object[] getField(Object obj, String elFieldName,
                                     boolean resolveParent) throws IllegalAccessException,
            IllegalArgumentException, NoSuchFieldException {
        if (obj == null) {
            return null;
        }
        String[] fieldNames = elFieldName.split("[.]");
        Object targetObj = obj;
        Class<?> targetClass = targetObj.getClass();
        Object val = null;
        int i = 0;
        Field field = null;
        Object[] rs = new Object[2];
        for (String fName : fieldNames) {
            i++;
            field = getField_(targetClass, fName, resolveParent);
            field.setAccessible(true);
            rs[0] = field;
            rs[1] = targetObj;
            val = field.get(targetObj);
            if (val == null) {
                if (i < fieldNames.length) {
                    throw new IllegalAccessException(
                            "can not getFieldValue as field '" + fName
                                    + "' value is null in '"
                                    + targetClass.getName() + "'");
                }
                break;
            }
            targetObj = val;
            targetClass = targetObj.getClass();
        }
        return rs;
    }

    public static Field getField_(Class<?> targetClass, String fieldName,
                                  boolean resolveParent) throws IllegalAccessException,
            IllegalArgumentException, NoSuchFieldException {
        NoSuchFieldException noSuchFieldExceptionOccor = null;
        Field rsField = null;
        try {
            Field field = targetClass.getDeclaredField(fieldName);
            rsField = field;
            if (!resolveParent) {
                field.setAccessible(true);
                return field;
            }
        } catch (NoSuchFieldException e) {
            noSuchFieldExceptionOccor = e;
        }
        if (noSuchFieldExceptionOccor != null) {
            if (resolveParent) {
                while (true) {
                    targetClass = targetClass.getSuperclass();
                    if (targetClass == null) {
                        break;
                    }
                    try {
                        Field field = targetClass.getDeclaredField(fieldName);
                        field.setAccessible(true);
                        rsField = field;
                        break;
                    } catch (NoSuchFieldException e) {
                        if (targetClass.getSuperclass() == null) {
                            throw e;
                        }
                    }
                }
            } else {
                throw noSuchFieldExceptionOccor;
            }
        }
        return rsField;
    }
}
