package cn.ninegame.genericframework.tools;

import android.annotation.TargetApi;
import android.os.Build;
import android.text.TextUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import cn.ninegame.genericframework.basic.Framework;

/**
 * Created by Administrator on 2017-07-31.
 */

public class SoLoader {

    public static final File LIB_DIR;

    static {
        LIB_DIR = new File(Framework.STORAGE_LOCATION,"lib");
        if(!LIB_DIR.exists()){
            LIB_DIR.mkdirs();
        }
    }

    public static void loadLibrary(String libName){
        if(TextUtils.isEmpty(libName)){
            throw new RuntimeException("can not load library without a name");
        }
        try{
            Runtime.getRuntime().loadLibrary(libName);
        }catch(UnsatisfiedLinkError error){
            if(!LIB_DIR.exists()){
                LIB_DIR.mkdirs();
            }
            if(!LIB_DIR.exists()){
                throw error;
            }
            String soName = System.mapLibraryName(libName);
            if(supportArmeabi()){
                File soFile = null;
                if(findLocalLibrary(soName) == null){
                    extractSoFromApk(soName);
                }
                if((soFile=findLocalLibrary(soName))!=null){
                    System.load(soFile.getAbsolutePath());
                }else{
                    throw error;
                }
            }
        }
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private static boolean supportArmeabi(){
        if(Build.VERSION.SDK_INT>=21) {
            String[] abis = Build.SUPPORTED_ABIS;
            if (abis != null) {
                for (String abi : abis) {
                    if (abi.equalsIgnoreCase("armeabi")) {
                        return true;
                    }
                }
            }
        }else{
            if(Build.CPU_ABI.contains("armeabi") || Build.CPU_ABI2.contains("armeabi")){
                return true;
            }
        }

        return false;
    }

    private static File findLocalLibrary(String libraryName){
        File soFile;
        if(LIB_DIR.exists() && (soFile=new File(LIB_DIR,libraryName)).exists()){
            return soFile;
        }else{
            return null;
        }
    }

    public static synchronized void extractSoFromApk(ZipFile zipFile,String[] primaryCpuAbis){
        if(null==primaryCpuAbis || primaryCpuAbis.length==0) return;
        GundamFileLock.getInstance().LockExclusive(LIB_DIR);
        try {
            if(zipFile==null) return;
            Enumeration<? extends ZipEntry> entries = zipFile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry targetEntry = entries.nextElement();
                if (targetEntry.getName().startsWith("lib/"+primaryCpuAbis[0]+"/")) {
                    int retryCount = 2;
                    do {
                        --retryCount;
                        String entryName=  targetEntry.getName();
                        String soName = entryName.replace("lib/"+primaryCpuAbis[0]+"/","");
                        if(findLocalLibrary(soName)!=null){
                            break;
                        }
                        File targetFile = new File(LIB_DIR, soName);
                        File targetFileTmp = new File(LIB_DIR,soName+".tmp");
                        if(!targetFileTmp.exists() || targetFileTmp.length()!=targetEntry.getSize()) {
                            if(targetFileTmp.exists()){
                                targetFileTmp.delete();
                            }
                            try{
                                BufferedOutputStream bos = new BufferedOutputStream(
                                        new FileOutputStream(targetFileTmp.getAbsolutePath()));
                                BufferedInputStream bi = new BufferedInputStream(zipFile.getInputStream(targetEntry));
                                byte[] readContent = new byte[512];
                                int readCount = bi.read(readContent);
                                while (readCount != -1) {
                                    bos.write(readContent, 0, readCount);
                                    readCount = bi.read(readContent);
                                }
                                try {
                                    bos.close();
                                    bi.close();
                                } catch (Throwable e) {
                                }
                            }catch (Exception ex){
                                ex.printStackTrace();
                            }
                        }
                        if (targetFileTmp.exists()) {
                            if(targetFileTmp.length() == targetEntry.getSize()){
                                targetFileTmp.renameTo(targetFile);
                                if(!targetFile.exists()){
                                    targetFileTmp.renameTo(targetFile);
                                }
                                if(targetFile.exists()){
                                    break;
                                }
                            }else{
                                targetFileTmp.delete();
                            }
                        }
                    } while (retryCount > 0);
                }
            }
        }catch (Exception ex){
            ex.printStackTrace();
        }finally {
            GundamFileLock.getInstance().unLock(LIB_DIR);
        }
    }

    public static synchronized void extractSoFromApk(){
        GundamFileLock.getInstance().LockExclusive(LIB_DIR);
        try {
            ZipFile zipFile = ApkUtils.getApk();
            if(zipFile==null) return;
            Enumeration<? extends ZipEntry> entries = zipFile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry targetEntry = entries.nextElement();
                if (targetEntry.getName().startsWith("lib/armeabi/")) {
                    int retryCount = 2;
                    do {
                        --retryCount;
                        String entryName=  targetEntry.getName();
                        String soName = entryName.replace("lib/armeabi/","");
                        if(findLocalLibrary(soName)!=null){
                            break;
                        }
                        File targetFile = new File(LIB_DIR, soName);
                        File targetFileTmp = new File(LIB_DIR,soName+".tmp");
                        if(!targetFileTmp.exists() || targetFileTmp.length()!=targetEntry.getSize()) {
                            if(targetFileTmp.exists()){
                                targetFileTmp.delete();
                            }
                            try{
                                BufferedOutputStream bos = new BufferedOutputStream(
                                        new FileOutputStream(targetFileTmp.getAbsolutePath()));
                                BufferedInputStream bi = new BufferedInputStream(zipFile.getInputStream(targetEntry));
                                byte[] readContent = new byte[512];
                                int readCount = bi.read(readContent);
                                while (readCount != -1) {
                                    bos.write(readContent, 0, readCount);
                                    readCount = bi.read(readContent);
                                }
                                try {
                                    bos.close();
                                    bi.close();
                                } catch (Throwable e) {
                                }
                            }catch (Exception ex){
                                ex.printStackTrace();
                            }
                        }
                        if (targetFileTmp.exists()) {
                            if(targetFileTmp.length() == targetEntry.getSize()){
                                targetFileTmp.renameTo(targetFile);
                                if(!targetFile.exists()){
                                    targetFileTmp.renameTo(targetFile);
                                }
                                if(targetFile.exists()){
                                    break;
                                }
                            }else{
                                targetFileTmp.delete();
                            }
                        }
                    } while (retryCount > 0);
                }
            }
        }catch (Exception ex){
            ex.printStackTrace();
        }finally {
            GundamFileLock.getInstance().unLock(LIB_DIR);
        }
    }

    private static synchronized void extractSoFromApk(String soName){
        GundamFileLock.getInstance().LockExclusive(LIB_DIR);
        if(findLocalLibrary(soName)!=null){
            return;
        }
        try {
            int retryCount = 2;
            do {
                --retryCount;
                String entryName = String.format("lib/armeabi/%s", soName);
                ZipEntry targetEntry = null;
                if (ApkUtils.getApk() != null && (targetEntry = ApkUtils.getApk().getEntry(entryName)) != null) {
                    File targetFile = new File(LIB_DIR, soName);
                    File targetFileTmp = new File(LIB_DIR,soName+".tmp");
                    if(!targetFileTmp.exists() || targetFileTmp.length()!=targetEntry.getSize()) {
                        if(targetFileTmp.exists()){
                            targetFileTmp.delete();
                        }
                        BufferedOutputStream bos = new BufferedOutputStream(
                                new FileOutputStream(targetFileTmp.getAbsolutePath()));
                        BufferedInputStream bi = new BufferedInputStream(ApkUtils.getApk().getInputStream(targetEntry));
                        byte[] readContent = new byte[512];
                        int readCount = bi.read(readContent);
                        while (readCount != -1) {
                            bos.write(readContent, 0, readCount);
                            readCount = bi.read(readContent);
                        }
                        try {
                            bos.close();
                            bi.close();
                        } catch (Throwable e) {
                        }
                    }
                    if (targetFileTmp.exists()) {
                        if(targetFileTmp.length() == targetEntry.getSize()){
                            targetFileTmp.renameTo(targetFile);
                            if(!targetFile.exists()){
                                targetFileTmp.renameTo(targetFile);
                            }
                            if(targetFile.exists()){
                                break;
                            }
                        }else{
                            targetFileTmp.delete();
                        }
                    }

                }
            } while (retryCount > 0);
        }catch(Exception e){
            e.printStackTrace();
        }finally {
            GundamFileLock.getInstance().unLock(LIB_DIR);
        }

    }
}