package cn.ninegame.genericframework.ui;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.util.Log;
import android.view.LayoutInflater;
import cn.ninegame.genericframework.tools.ReflectHelper;

import java.lang.reflect.Field;

/**
 *
 * Something like Dialog, needs the activity but also needs module context, can use this.
 * 
 * Created by chenkai on 2016/3/3.
 */
public class ActivityContextWrapper extends ContextWrapper {
    private final Context mModuleContext;
    private final Activity mActivity;

    public ActivityContextWrapper(Activity activity, Context moduleContext) {
        super(activity);
        mModuleContext = moduleContext;
        mActivity = activity;
        // apply activity theme into module context's theme
        try {
            Field themeField = ReflectHelper.findField(activity, "mThemeResource");
            themeField.setAccessible(true);
            getTheme().applyStyle(themeField.getInt(activity), true);
        } catch (Exception e) {
            Log.e("ActivityContextWrapper", "", e);
        }
    }

    public boolean isSameActivity(Activity activity) {
        return activity == mActivity;
    }

    @Override
    public Object getSystemService(String name) {
        if (Context.WINDOW_SERVICE.equals(name)) {
            return getBaseContext().getSystemService(name);
        } else if (Context.LAYOUT_INFLATER_SERVICE.equals(name)) {
            return LayoutInflater.from(mModuleContext).cloneInContext(this);
        } else {
            return mModuleContext.getSystemService(name);
        }
    }

    @Override
    public AssetManager getAssets() {
        return getResources().getAssets();
    }

    @Override
    public Resources getResources() {
        return mModuleContext.getResources();
    }

    @Override
    public Resources.Theme getTheme() {
        return mModuleContext.getTheme();
    }
}
