package cn.ninegame.genericframework.ui;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.FragmentTransaction;
import android.view.Window;

import cn.ninegame.genericframework.basic.Environment;
import cn.ninegame.genericframework.basic.FrameworkFacade;
import cn.ninegame.genericframework.basic.IResultListener;
import cn.ninegame.genericframework.module.IModuleComponent;
import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.module.ModuleType;
import cn.ninegame.genericframework.tools.ReflectHelper;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2015-05-27
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
public class BaseDialogFragment extends DialogFragment implements IModuleComponent {

    private Bundle mBundle = new Bundle();

    private IResultListener mResultListener;
    protected Environment mEnv;
    private IModuleEntry mModuleEntry;
    private ActivityContextWrapper mActivityContextWrapper;

    @Override
    public Environment getEnvironment(){
        return mEnv;
    }

    @Override
    public void setEnvironment(Environment environment){
        mEnv = environment;
    }

    @Deprecated
    @Override
    public void setArguments(Bundle args) {
        super.setArguments(args);
    }

    public void setBundleArguments(Bundle bundle){
        mBundle = bundle;
    }

    public Bundle getBundleArguments(){
        return mBundle;
    }

    public String getName(){
        return this.getClass().getName();
    }

    public void onResult(Bundle result){
        if(mResultListener != null){
            mResultListener.onResult(result);
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (mEnv == null) {
            FrameworkFacade.getInstance().getEnvironment().reInitFragment(this);
        }
    }

    public void setResultListener(IResultListener listener) {
        this.mResultListener = listener;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        Dialog dialog = new Dialog(getContext(), getTheme());
        initDialogStyle(dialog);
        return dialog;
    }

    protected void initDialogStyle(Dialog dialog) {
        dialog.setCanceledOnTouchOutside(false);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
    }

    @Override
    public void setModuleEntry(IModuleEntry entry) {
        if (mModuleEntry != null) {
            throw new IllegalArgumentException("Module entry has already set!");
        }
        mModuleEntry = entry;
    }

    @Override
    public Context getContext() {
        if (mModuleEntry == null){
            throw new RuntimeException(String.format("getContext,  mModuleEntry is null, className = %s", getName()));
        }
        Activity activity = mEnv.getCurrentActivity();
        if (mModuleEntry.getModuleType() == ModuleType.LOCAL) {
            return activity;
        } else {
            ActivityContextWrapper wrapper = mActivityContextWrapper;
            if (wrapper == null || !wrapper.isSameActivity(activity)) {
                wrapper = new ActivityContextWrapper(activity, mModuleEntry.getContext());
                mActivityContextWrapper = wrapper;
            }
            return wrapper;
        }
    }

    @Override
    public String getModuleRootPath(){
        return mModuleEntry.getModuleRootPath();
    }

    @Override
    public String getModuleDataPath(){
        return mModuleEntry.getModuleDataPath();
    }

    @Override
    public String getModuleSoPath(){
        return mModuleEntry.getModuleSoPath();
    }

    @Override
    public int show(FragmentTransaction transaction, String tag) {
        try {
            ReflectHelper.setField(this, "mDismissed", false);
            ReflectHelper.setField(this, "mShownByMe", true);
            transaction.add(this, tag);
            ReflectHelper.setField(this, "mViewDestroyed", false);
            int backStackId = transaction.commitAllowingStateLoss();
            ReflectHelper.setField(this, "mBackStackId", backStackId);
            return backStackId;

        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        }

        return super.show(transaction, tag);
    }
}
