package cn.ninegame.genericframework.updatecenter;

import android.os.AsyncTask;
import android.text.TextUtils;
import android.util.Log;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2016 www.9game.cn All Rights Reserved
 * <p/>
 * Created by zhonghui on 2016-07-20.
 * Description :
 * ****************************************************************************
 */
public class ModuleUpdateBusiness{

    private static ModuleUpdateBusiness me;
    private AtomicBoolean mInitAtomic = new AtomicBoolean(false);
    private AtomicBoolean mQueryAtomic = new AtomicBoolean(false);
    private IModuleUpdRequest mModuleUpdateRequest;
    private HashMap<String,ModuleUpdHandle> mModuleHandles =new HashMap<String,ModuleUpdHandle>();

    public static ModuleUpdateBusiness getInstance(){
        if(me==null){
            me= new ModuleUpdateBusiness();
        }
        return me;
    }

    public void buildNetwork(IModuleUpdRequest moduleUpdateRequest){
        this.mModuleUpdateRequest  = moduleUpdateRequest;
    }

    public void registerModuleHandle(ModuleUpdHandle moduleHandles){
        if(!TextUtils.isEmpty(moduleHandles.getType()) && moduleHandles!=null){
            this.mModuleHandles.put(moduleHandles.getType(),moduleHandles);
        }
    }

    public void unRegisterModuleHandle(String type){
        if(!TextUtils.isEmpty(type)){
            this.mModuleHandles.remove(type);
        }
    }

    public void doneInit(){
        this.mInitAtomic.set(true);
    }

    public boolean getInitStatus(){
        return mInitAtomic.get();
    }

    public void queryNewModule(List<ModuleUpdInfo> moduleList) {
        if (mQueryAtomic.compareAndSet(false, true)) {
            QueryModuleInfoTask queryTask = new QueryModuleInfoTask();
            queryTask.execute(moduleList);
        }
    }

    public void queryNewModule(List<ModuleUpdInfo> moduleList,IModuleUpdCallback moduleUpdCallback) {
        if (mQueryAtomic.compareAndSet(false, true)) {
            Log.i("updatecenter","queryNewModule_QueryModuleInfoTask");
            QueryModuleInfoTask queryTask = new QueryModuleInfoTask(moduleUpdCallback);
            queryTask.execute(moduleList);
        }
    }

    public void processModuleUpdCallBack(ModuleUpdResponseData moduleDataObject){
        if(mModuleHandles!=null && mModuleHandles.size()>0 && moduleDataObject!=null){
            String type = moduleDataObject.getType();
            if(TextUtils.isEmpty(type)){
                return;
            }
            ModuleUpdHandle handle = mModuleHandles.get(type);
            if(handle!=null){
                handle.receiveModule(moduleDataObject);
            }
        }
    }

    private class QueryModuleInfoTask extends AsyncTask<List<ModuleUpdInfo>, Void, ModuleUpdResponseData> {

        private IModuleUpdCallback mIModuleUpdCallback;

        public QueryModuleInfoTask(IModuleUpdCallback moduleUpdCallback){
            this.mIModuleUpdCallback = moduleUpdCallback;
        }

        public QueryModuleInfoTask(){
        }

        @Override
        protected ModuleUpdResponseData doInBackground(List<ModuleUpdInfo>... lists) {
            try {
                Log.i("updatecenter","queryNewModule_QueryModuleInfoTask_doInBackground");
                ModuleUpdResponseData moduleUpdResponseData = null;
                if(lists!=null && lists.length>0){
                    Log.i("updatecenter","queryNewModule_QueryModuleInfoTask_uploadModuleInfo");
                    boolean bResult =  uploadModuleInfo(lists[0]);
                    Log.i("updatecenter","uploadModuleInfo result:" + (bResult?"true":"false"));
                    if(bResult){
                        moduleUpdResponseData = checkModuleUpdateInfo();
                    }
                }
                return moduleUpdResponseData;
            }finally {
                mQueryAtomic.set(false);
            }
        }

        @Override
        protected void onPostExecute(ModuleUpdResponseData moduleDataObject) {
            if(moduleDataObject!=null){
                Log.i("updatecenter","checkModuleUpdateInfo:" + moduleDataObject.getFileName()!=null?"":moduleDataObject.getFileName());
                if(mIModuleUpdCallback==null){
                    String type = moduleDataObject.getType();
                    if(TextUtils.isEmpty(type)){
                        return;
                    }
                    ModuleUpdHandle handle = mModuleHandles.get(type);
                    if(handle!=null){
                        handle.receiveModule(moduleDataObject);
                    }
                }else{
                    mIModuleUpdCallback.callBack(moduleDataObject);
                }
            }else{
                Log.i("updatecenter","checkModuleUpdateInfo: no module update");
            }
        }
    }

    public ModuleUpdResponseData checkModuleUpdateInfo(){
        if(mModuleUpdateRequest==null) {
            Log.i("updatecenter","checkModuleUpdateInfo=mModuleUpdateRequest=null");
            return null;
        }
        ModuleUpdResponse response = mModuleUpdateRequest.checkModule();
        if(response!=null && response.getState()!=null){
            Log.i("updatecenter","checkModule result:" + String.valueOf(response.getState().getCode()));
            if(response.getState().getCode()==2000000 && response.getData()!=null){
                return response.getData();
            }else{
                return null;
            }
        }else{
            return null;
        }
    }

    public boolean uploadModuleInfo(List<ModuleUpdInfo> udpUpdateInfo){
        if(mModuleUpdateRequest==null) {
            Log.i("updatecenter","mModuleUpdateRequest=uploadModuleInfo=null");
            return false;
        }
        ModuleUpdResponse response  = mModuleUpdateRequest.uploadModuleInfo(udpUpdateInfo);
        if(response!=null && response.getState()!=null){
            Log.i("updatecenter","response.getState()!=null");
            Log.i("updatecenter","response.getState().getCode()="+String.valueOf(response.getState().getCode()));
            if(response.getState().getCode()==2000000){
                return true;
            }
        }else{
            Log.i("updatecenter","response==null");
        }
        return false;
    }

}
